##########################################################
##                                                      ##
## INSTALL PROCEDURE FOR CMF/PLONE TYPES                ##
##                                                      ##
## This can be shared across products ! :-)             ##
##                                                      ##
##########################################################


from Products.CMFBibliography import install_globals
from Products.CMFBibliography import BibliographyFolder
from Products.CMFBibliography import BibliographyEntry
from Products.CMFBibliography.global_symbols import *

from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName

from AccessControl.Permission import pname
import Globals

from cStringIO import StringIO
import string


fti_list = (
    BibliographyFolder.factory_type_information,
    BibliographyEntry.factory_type_information,
    )
types_list = (
    "BibliographyFolder",
    "BibliographyEntry",
    )
skin_name = "bibliography"
perms_list = (BibliographyFolder_editPermission,
              BibliographyFolder_addPermission,
              BibliographyEntry_addPermission, )


def installSubSkin(self, out, skinFolder):
    """ Install a subskin, i.e. a folder/directoryview.
    """
    skinsTool = getToolByName(self, 'portal_skins')

    for skin in skinsTool.getSkinSelections():
        path = skinsTool.getSkinPath(skin)
        path = map( string.strip, string.split( path,',' ) )
        if not skinFolder in path:
            try:
                path.insert( path.index( 'custom')+1, skinFolder )
            except ValueError:
                path.append(skinFolder)
            path = string.join( path, ', ' )
            skinsTool.addSkinSelection( skin, path )
            out.write('Subskin successfully installed into %s.\n' % skin)    
        else:
            out.write('*** Subskin was already installed into %s.\n' % skin) 


def setupTypesandSkins(self, out):
    typesTool = getToolByName(self, 'portal_types')
    skinsTool = getToolByName(self, 'portal_skins')
    
    # Former types deletion (added by PJG)
    for f in fti_list:
        if f['id'] in typesTool.objectIds():
            out.write('*** Object "%s" already existed in the types tool \
            => deleting\n' % (f['id']))
            
            typesTool._delObject(f['id'])

    # Type re-creation
    for f in fti_list:
        cfm = apply(ContentFactoryMetadata, (), f)
        typesTool._setObject(f['id'], cfm)
        out.write('Type "%s" registered with the types tool\n' % (f['id']))

        
    # Install de chaque nouvelle subskin/layer
    try:  
        addDirectoryViews(skinsTool, 'skins', install_globals)
        out.write( "Added directory views to portal_skins.\n" )
    except:
        out.write( '*** Unable to add directory views to portal_skins.\n')

    # Param de chaque nouvelle subskin/layer
    installSubSkin(self, out, skin_name)

def addToCatalog(self, out):
    ct = getToolByName(self, 'portal_catalog')

    newFieldIndexes = ['Authors', 'publication_year']
    newSchemaEntries = ['Authors', 'publication_year', 'Source']

    for idx in newFieldIndexes:
        if idx in ct.indexes():
            out.write("Found the '%s' index in the catalog.\n" % idx)
        else:
            ct.addIndex(idx, 'FieldIndex')
            out.write("Added a field index '%s' to the catalog.\n" % idx)
            
    for entry in newSchemaEntries:
        if entry in ct.schema():
            out.write("Found '%s' in the catalog meta data.\n" % entry)
        else:
            ct.addColumn(entry)
            out.write("Added '%s' to the catalog meta data.\n" % entry)


def addPortalTab(self, out):
    at = getToolByName(self, 'portal_actions')

    if 'bibliography' not in [a.getId() for a in at.listActions()]:
        at.addAction(id = 'bibliography',
                     name = 'Bibliography',
                     action = 'string:$portal_url/bibliography_view',
                     condition = None,
                     permission = 'View',
                     category = 'portal_tabs'
                     )
    
        out.write("Added 'Bibliography' to the portal tabs.\n")
    else:
        out.write("'Bibliography' tab already defined.\n")
    
def addNavigationTransitions(self):
    portal_prop = getToolByName(self, 'portal_properties')
    form = portal_prop.form_properties
    nav = portal_prop.navigation_properties

    for ctype in types_list:
        lowertype = ctype.lower()

        # The mapping var provides a way to make
        # properties creation / updating easier
        mapping = [
            ('%s_editForm' % (ctype), 'validate_id,validate_%s_edit' % (ctype), ),
            ]
        
        for (name, value) in mapping:
            try:
                form._setProperty(name, value, 'string')
            except:
                form._updateProperty(name, value)

        mapping = [
            ('%s.%s_editForm.success' % (lowertype, ctype ), 'script:%s_edit' % (ctype), ),
            ('%s.%s_editForm.failure' % (lowertype, ctype ), '%s_editForm' % (ctype), ),
            ('%s.%s_edit.success' % (lowertype, ctype ), 'action:view', ),
            ('%s.%s_edit.failure' % (lowertype, ctype ), 'action:edit', ),
            ]

        for (name, value) in mapping:
            try:
                nav._setProperty(name, value, 'string')
            except:
                nav._updateProperty(name, value)


    ## One can add additional portal properties here. Here come some examples :
##    form._setProperty('breve_images_form', 'validate_breve_edit', 'string')
##    form._setProperty('breve_textattachments_form', 'validate_breve_edit', 'string')
##    form._setProperty('breve_mmattachments_form', 'validate_breve_edit', 'string')

##    nav._setProperty('breve.breve_images_form.success', 'script:breve_edit', 'string')
##    nav._setProperty('breve.breve_images_form.failure', 'breve_images_form', 'string')

##    nav._setProperty('breve.breve_textattachments_form.success', 'script:breve_edit', 'string')
##    nav._setProperty('breve.breve_textattachments_form.failure', 'breve_textattachments_form', 'string')

##    nav._setProperty('breve.breve_mmattachments_form.success', 'script:breve_edit', 'string')
##    nav._setProperty('breve.breve_mmattachments_form.failure', 'breve_mmattachments_form', 'string')



def setPermissions(self, out):
    """
    setPermissions(self, out) => Set standard permissions / roles
    """
    p = self

    # As a default behavior, newly-created permissions are granted to owner and manager.
    # To change this, just comment this code and grab back the code commented below to
    # make it suit your needs.
    for perm in perms_list:
        p.manage_permission(
            perm,
            ('Manager', 'Owner'),
            acquire = 1
            )

##    # EDIT FROM HERE...
##    p.manage_permission(
##        BibliographyFolder_editPermission,
##        ('Manager', 'Owner',),
##        acquire=1
##        )
##    p.manage_permission(
##        BibliographyFolder_addPermission,
##        ('Manager', 'Owner',),
##        acquire=1
##        )
##    # ...TO HERE
    out.write("Reseted default permissions\n")
                

def install(self):
    out=StringIO()
    setPermissions(self, out)
    setupTypesandSkins(self, out)
    addToCatalog(self, out)
    addPortalTab(self,out)
    addNavigationTransitions(self)
    out.write('\nInstallation completed.\n')
    return out.getvalue()

